import React from "react";
import { emptyReport } from "@/provider/dummy/report";
import {
    Button,
    FormControl,
    FormLabel,
    Grid,
    Typography,
    FormGroup,
    FormControlLabel,
    Checkbox,
    Radio,
    InputLabel,
    Select,
    MenuItem
} from "@material-ui/core";
import { Actions } from "@/components";
import { Link as RouterLink } from "react-router-dom";
import { route } from "@/routing";
import { useTranslation } from "react-i18next";
import { MultiChoiceField, Report, ReportField, SelectField, SingleChoiceField } from "@/data/report";
import { TextField as TextFieldFormik } from "formik-material-ui";
import { Field, Form, Formik, useFormik, useFormikContext } from "formik";
import { Multilingual } from "@/data";
import { Transformer } from "@/serialization";

export type ReportFieldProps<TField = ReportField> = {
    field: TField;
}

const CustomField = ({ field, ...props }: ReportFieldProps) => {
    switch (field.type) {
        case "text":
            return <CustomField.Text {...props} field={ field } />
        case "single-choice":
        case "multi-choice":
            return <CustomField.Choice {...props} field={ field }/>
        case "select":
            return <CustomField.Select {...props} field={ field }/>
    }
}

CustomField.Text = ({ field }: ReportFieldProps) => {
    return <>
        <Field label={ field.label.pl } name={ field.name } fullWidth component={ TextFieldFormik }/>
        <Typography variant="caption" color="textSecondary" dangerouslySetInnerHTML={{ __html: field.description.pl }}/>
    </>
}

CustomField.Select = ({ field }: ReportFieldProps<SelectField>) => {
    const { t } = useTranslation();
    const id = `custom-field-${field.name}`;
    const { values, setFieldValue } = useFormikContext<any>();

    const value = values[field.name];

    return <FormControl variant="outlined">
        <InputLabel htmlFor={id}>{ field.label.pl }</InputLabel>
        <Select label={ field.label.pl } name={ field.name } id={id} value={ value } onChange={ ({ target }) => setFieldValue(field.name, target.value, false) }>
            { field.choices.map(choice => <MenuItem value={ choice as any }>{ choice.pl }</MenuItem>) }
        </Select>
        <Typography variant="caption" color="textSecondary" dangerouslySetInnerHTML={{ __html: field.description.pl }}/>
    </FormControl>
}

CustomField.Choice = ({ field }: ReportFieldProps<SingleChoiceField | MultiChoiceField>) => {
    const { t } = useTranslation();
    const { values, setFieldValue } = useFormikContext<any>();

    const value = values[field.name];

    const isSelected = field.type == 'single-choice'
        ? (checked: Multilingual<string>) => value == checked
        : (checked: Multilingual<string>) => (value || []).includes(checked)

    const handleChange = field.type == 'single-choice'
        ? (choice: Multilingual<string>) => () => setFieldValue(field.name, choice, false)
        : (choice: Multilingual<string>) => () => {
            const current = value || [];
            setFieldValue(field.name, !current.includes(choice) ? [ ...current, choice ] : current.filter((c: Multilingual<string>) => c != choice), false);
        }

    const Component = field.type == 'single-choice' ? Radio : Checkbox;

    return <FormControl component="fieldset">
        <FormLabel component="legend">{ field.label.pl }</FormLabel>
        <FormGroup>
            { field.choices.map(choice => <FormControlLabel
                control={ <Component checked={ isSelected(choice) } onChange={ handleChange(choice) }/> }
                label={ choice.pl }
            />) }
        </FormGroup>
        <Typography variant="caption" color="textSecondary" dangerouslySetInnerHTML={{ __html: field.description.pl }}/>
    </FormControl>
}

export type ReportFormValues = { [field: string]: any };

const reportToFormValuesTransformer: Transformer<Report, ReportFormValues, { report: Report }> = {
    reverseTransform(subject: ReportFormValues, context: { report: Report }): Report {
        return { ...context.report };
    },
    transform(subject: Report, context: undefined): ReportFormValues {
        return Object.fromEntries(subject.fields.map(field => [ field.name, field.value ]));
    }
}

export default function ReportForm() {
    const report = emptyReport;
    const { t } = useTranslation();

    const handleSubmit = async () => {};

    return <Formik initialValues={ reportToFormValuesTransformer.transform(report) } onSubmit={ handleSubmit }>
        { ({ submitForm }) => <Form>
            <Grid container>
                <Grid item xs={12}>
                    <Typography variant="body1" component="p">{ t('forms.report.instructions') }</Typography>
                </Grid>
                { report.fields.map(field => <Grid item xs={12}><CustomField field={ field }/></Grid>) }
                <Grid item xs={12}>
                    <Actions>
                        <Button variant="contained" color="primary" onClick={ submitForm }>
                            { t('confirm') }
                        </Button>

                        <Button component={ RouterLink } to={ route("home") }>
                            { t('go-back') }
                        </Button>
                    </Actions>
                </Grid>
            </Grid>
        </Form> }
    </Formik>
}

